@@ -1,5 +1,5 @@
 __title__ = 'wfuzz'
-__version__ = "2.3.4"
+__version__ = "2.4"
 __build__ = 0x023000
 __author__ = 'Xavier Mendez'
 __license__ = 'GPL 2.0'
@@ -23,6 +23,9 @@ try:
     if "openssl".lower() not in pycurl.version.lower():
         print("\nWarning: Pycurl is not compiled against Openssl. Wfuzz might not work correctly when fuzzing SSL sites. Check Wfuzz's documentation for more information.\n")
 
+    if not hasattr(pycurl, "CONNECT_TO"):
+        print("\nWarning: Pycurl and/or libcurl version is old. CONNECT_TO option is missing. Wfuzz --ip option will not be available.\n")
+
     if not hasattr(pycurl, "PATH_AS_IS"):
         print("\nWarning: Pycurl and/or libcurl version is old. PATH_AS_IS option is missing. Wfuzz might not correctly fuzz URLS with '..'.\n")
 
