@@ -5,6 +5,7 @@ from wfuzz.externals.moduleman.plugin import moduleman_plugin
 from wfuzz.exception import FuzzExceptBadFile
 from wfuzz.fuzzobjects import FuzzResult
 from wfuzz.plugin_api.base import BasePayload
+from wfuzz.utils import rgetattr
 
 
 @moduleman_plugin
@@ -45,7 +46,7 @@ class wfuzzp(BasePayload):
     def __next__(self):
         next_item = next(self._it)
 
-        return next_item if not self.attr else next_item.get_field(self.attr)
+        return next_item if not self.attr else rgetattr(next_item, self.attr)
 
     def _gen_wfuzz(self, output_fn):
         try:
